package jez04.structure.test;

import static org.hamcrest.MatcherAssert.assertThat;

import java.net.URISyntaxException;
import java.nio.file.Path;
import java.util.Objects;
import java.util.stream.Stream;

import org.hamcrest.Matchers;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;

import cz.vsb.fei.kelvin.unittest.SrcContains;
import cz.vsb.fei.kelvin.unittest.StructureHelper;
import cz.vsb.fei.kelvin.unittest.TextFileContains;
import cz.vsb.fei.kelvin.unittest.XmlFileContains;

class ClassStructureTest {

	StructureHelper helper = StructureHelper.getInstance(ClassStructureTest.class);
	
	@Test
	void lombokAsDependencyTest() throws URISyntaxException {
		XmlFileContains xmlFileContains = new XmlFileContains("pom.xml", "/project/dependencies/dependency/artifactId[text() = 'lombok']");
		Path root = TextFileContains.getProjectRoot(getClass()); 
		assertThat(root, xmlFileContains);
	}

	@Test
	void lombokAsAnnotationProcessorTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()), new XmlFileContains("pom.xml", "/project/build/plugins/plugin/artifactId[text() = 'maven-compiler-plugin']"));
		assertThat(TextFileContains.getProjectRoot(getClass()), new XmlFileContains("pom.xml", "/project/build/plugins/plugin/configuration/annotationProcessorPaths/path/artifactId[text() = 'lombok']"));
	}

	@Test
	void moduleInfoTest() throws URISyntaxException {
		assertThat(TextFileContains.getProjectRoot(getClass()), new TextFileContains("module-info.java", "lombok;"));
	}

	@CsvSource({
		"Config,@Getter,1",
		"Config,@.*Builder,1",
		"Config,@.*Default,3"
		})
	@ParameterizedTest(name = "useLombokTest in {1} annotation {2}")
	void useLombokConfigTest(String className, String text, int count) throws URISyntaxException, ClassNotFoundException {
		Class<?> config = helper.getClass(className);
		assertThat(config, new SrcContains(text).count(count));
	}

	@CsvSource({
		"Score,@Setter,1",
		"Score,@Getter,1",
		"Score,@AllArgsConstructor,1",
		"Score,@EqualsAndHashCode,1",
		"Score,@ToString,1"
		})
	@ParameterizedTest(name = "useLombokTest in {1} annotation {2}")
	void useLombokOrDataTest(String className, String text, int count) throws URISyntaxException, ClassNotFoundException {
		Class<?> config = helper.getClass(className);
		assertThat(config, Matchers.anyOf(new SrcContains(text).count(count),
				new SrcContains("@Data").count(1)));
	}
	
	@Test
	void recordTest() throws URISyntaxException {
		
		long recordCount = Stream.concat(
				helper.getAllNonInnerClasses().stream(),
				helper.getAllInnerClasses().stream())
				.filter(className -> !className.startsWith("javafx")).map(className -> {
					try {
						return helper.getClassDirectly(className);
					} catch (ClassNotFoundException e) {
						e.printStackTrace();
						return null;
					}
				}).filter(Objects::nonNull).filter(clazz -> clazz.isRecord()).count();
		assertThat(recordCount, Matchers.greaterThanOrEqualTo(1L));
		
	}
	

}
